@@ -14,9 +14,7 @@
   General Public License for more details.
 
   You should have received a copy of the GNU Lesser General Public License
-  along with PulseAudio; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-  USA.
+  along with PulseAudio; if not, see <http://www.gnu.org/licenses/>.
 ***/
 
 #ifdef HAVE_CONFIG_H
@@ -35,6 +33,7 @@
 #include <pulsecore/hook-list.h>
 #include <pulsecore/sink-input.h>
 #include <pulsecore/modargs.h>
+#include <pulsecore/proplist-util.h>
 
 #include "module-filter-apply-symdef.h"
 
@@ -43,7 +42,7 @@
 PA_MODULE_AUTHOR("Colin Guthrie");
 PA_MODULE_DESCRIPTION("Load filter sinks automatically when needed");
 PA_MODULE_VERSION(PACKAGE_VERSION);
-PA_MODULE_LOAD_ONCE(TRUE);
+PA_MODULE_LOAD_ONCE(true);
 PA_MODULE_USAGE(_("autoclean=<automatically unload unused filters?>"));
 
 static const char* const valid_modargs[] = {
@@ -51,49 +50,41 @@ static const char* const valid_modargs[] = {
     NULL
 };
 
-#define DEFAULT_AUTOCLEAN TRUE
+#define DEFAULT_AUTOCLEAN true
 #define HOUSEKEEPING_INTERVAL (10 * PA_USEC_PER_SEC)
 
 struct filter {
     char *name;
     uint32_t module_index;
-    pa_bool_t is_sink;
-    pa_object *parent_obj;      /* source or sink that the filter is connected to */
-    pa_object *obj;             /* source or sink of the filter */
+    pa_sink *sink;
+    pa_sink *sink_master;
+    pa_source *source;
+    pa_source *source_master;
 };
 
 struct userdata {
     pa_core *core;
     pa_hashmap *filters;
-    pa_hook_slot
-        *sink_input_put_slot,
-        *sink_input_move_finish_slot,
-        *sink_input_proplist_slot,
-        *sink_input_unlink_slot,
-        *sink_unlink_slot,
-        *source_output_put_slot,
-        *source_output_move_finish_slot,
-        *source_output_proplist_slot,
-        *source_output_unlink_slot,
-        *source_unlink_slot;
-    pa_bool_t autoclean;
+    bool autoclean;
     pa_time_event *housekeeping_time_event;
 };
 
 static unsigned filter_hash(const void *p) {
     const struct filter *f = p;
 
-    if (f->is_sink)
-        return (unsigned) (PA_SINK(f->parent_obj)->index + pa_idxset_string_hash_func(f->name));
+    if (f->sink_master && !f->source_master)
+        return (unsigned) (f->sink_master->index + pa_idxset_string_hash_func(f->name));
+    else if (!f->sink_master && f->source_master)
+        return (unsigned) ((f->source_master->index << 16) + pa_idxset_string_hash_func(f->name));
     else
-        return (unsigned) ((PA_SOURCE(f->parent_obj)->index << 16) + pa_idxset_string_hash_func(f->name));
+        return (unsigned) (f->sink_master->index + (f->source_master->index << 16) + pa_idxset_string_hash_func(f->name));
 }
 
 static int filter_compare(const void *a, const void *b) {
     const struct filter *fa = a, *fb = b;
     int r;
 
-    if (fa->parent_obj != fb->parent_obj)
+    if (fa->sink_master != fb->sink_master || fa->source_master != fb->source_master)
         return 1;
     if ((r = strcmp(fa->name, fb->name)))
         return r;
@@ -101,15 +92,19 @@ static int filter_compare(const void *a, const void *b) {
     return 0;
 }
 
-static struct filter *filter_new(const char *name, pa_object* parent_obj, pa_bool_t is_sink) {
+static struct filter *filter_new(const char *name, pa_sink *sink, pa_source *source) {
     struct filter *f;
 
+    pa_assert(sink || source);
+
     f = pa_xnew(struct filter, 1);
     f->name = pa_xstrdup(name);
-    pa_assert_se(f->parent_obj = parent_obj);
-    f->is_sink = is_sink;
+    f->sink_master = sink;
+    f->source_master = source;
     f->module_index = PA_INVALID_INDEX;
-    f->obj = NULL;
+    f->sink = NULL;
+    f->source = NULL;
+
     return f;
 }
 
@@ -120,7 +115,7 @@ static void filter_free(struct filter *f) {
     pa_xfree(f);
 }
 
-static const char* should_filter(pa_object *o, pa_bool_t is_sink_input) {
+static const char* should_filter(pa_object *o, bool is_sink_input) {
     const char *apply;
     pa_proplist *pl;
 
@@ -140,12 +135,99 @@ static const char* should_filter(pa_object *o, pa_bool_t is_sink_input) {
     return NULL;
 }
 
-static pa_bool_t nothing_attached(pa_object *obj, pa_bool_t is_sink)
-{
-    if (is_sink)
-        return pa_idxset_isempty(PA_SINK(obj)->inputs);
+static bool should_group_filter(struct filter *filter) {
+    return pa_streq(filter->name, "echo-cancel");
+}
+
+static char* get_group(pa_object *o, bool is_sink_input) {
+    pa_proplist *pl;
+
+    if (is_sink_input)
+        pl = PA_SINK_INPUT(o)->proplist;
     else
-        return pa_idxset_isempty(PA_SOURCE(obj)->outputs);
+        pl = PA_SOURCE_OUTPUT(o)->proplist;
+
+    /* There's a bit of cleverness here -- the second argument ensures that we
+     * only group streams that require the same filter */
+    return pa_proplist_get_stream_group(pl, pa_proplist_gets(pl, PA_PROP_FILTER_APPLY), NULL);
+}
+
+/* For filters that apply on a source-output/sink-input pair, this finds the
+ * master sink if we know the master source, or vice versa. It does this by
+ * looking up streams that belong to the same stream group as the original
+ * object. The idea is that streams from the sam group are always routed
+ * together. */
+static bool find_paired_master(struct userdata *u, struct filter *filter, pa_object *o, bool is_sink_input) {
+    char *group;
+
+    if ((group = get_group(o, is_sink_input))) {
+        uint32_t idx;
+        char *g;
+        char *module_name = pa_sprintf_malloc("module-%s", filter->name);
+
+        if (is_sink_input) {
+            pa_source_output *so;
+
+            PA_IDXSET_FOREACH(so, u->core->source_outputs, idx) {
+                g = get_group(PA_OBJECT(so), false);
+
+                if (pa_streq(g, group)) {
+                    if (pa_streq(module_name, so->source->module->name)) {
+                        /* Make sure we're not routing to another instance of
+                         * the same filter. */
+                        filter->source_master = so->source->output_from_master->source;
+                    } else {
+                        filter->source_master = so->source;
+                    }
+
+                    pa_xfree(g);
+                    break;
+                }
+
+                pa_xfree (g);
+            }
+        } else {
+            pa_sink_input *si;
+
+            PA_IDXSET_FOREACH(si, u->core->sink_inputs, idx) {
+                g = get_group(PA_OBJECT(si), true);
+
+                if (pa_streq(g, group)) {
+                    if (pa_streq(module_name, si->sink->module->name)) {
+                        /* Make sure we're not routing to another instance of
+                         * the same filter. */
+                        filter->sink_master = si->sink->input_to_master->sink;
+                    } else {
+                        filter->sink_master = si->sink;
+                    }
+
+                    pa_xfree(g);
+                    break;
+                }
+
+                pa_xfree(g);
+            }
+        }
+
+        pa_xfree(group);
+        pa_xfree(module_name);
+
+        if (!filter->sink_master || !filter->source_master)
+            return false;
+    }
+
+    return true;
+}
+
+static bool nothing_attached(struct filter *f) {
+    bool no_si = true, no_so = true;
+
+    if (f->sink)
+        no_si = pa_idxset_isempty(f->sink->inputs);
+    if (f->source)
+        no_so = pa_idxset_isempty(f->source->outputs);
+
+    return no_si && no_so;
 }
 
 static void housekeeping_time_callback(pa_mainloop_api*a, pa_time_event* e, const struct timeval *t, void *userdata) {
@@ -162,14 +244,14 @@ static void housekeeping_time_callback(pa_mainloop_api*a, pa_time_event* e, cons
     u->housekeeping_time_event = NULL;
 
     PA_HASHMAP_FOREACH(filter, u->filters, state) {
-        if (filter->obj && nothing_attached(filter->obj, filter->is_sink)) {
+        if (nothing_attached(filter)) {
             uint32_t idx;
 
             pa_log_debug("Detected filter %s as no longer used. Unloading.", filter->name);
             idx = filter->module_index;
             pa_hashmap_remove(u->filters, filter);
             filter_free(filter);
-            pa_module_unload_request_by_index(u->core, idx, TRUE);
+            pa_module_unload_request_by_index(u->core, idx, true);
         }
     }
 
@@ -188,14 +270,14 @@ static void trigger_housekeeping(struct userdata *u) {
     u->housekeeping_time_event = pa_core_rttime_new(u->core, pa_rtclock_now() + HOUSEKEEPING_INTERVAL, housekeeping_time_callback, u);
 }
 
-static int do_move(pa_object *obj, pa_object *parent, pa_bool_t restore, pa_bool_t is_input) {
+static int do_move(pa_object *obj, pa_object *parent, bool restore, bool is_input) {
     if (is_input)
         return pa_sink_input_move_to(PA_SINK_INPUT(obj), PA_SINK(parent), restore);
     else
         return pa_source_output_move_to(PA_SOURCE_OUTPUT(obj), PA_SOURCE(parent), restore);
 }
 
-static void move_object_for_filter(pa_object *o, struct filter* filter, pa_bool_t restore, pa_bool_t is_sink_input) {
+static void move_object_for_filter(pa_object *o, struct filter* filter, bool restore, bool is_sink_input) {
     pa_object *parent;
     pa_proplist *pl;
     const char *name;
@@ -203,19 +285,23 @@ static void move_object_for_filter(pa_object *o, struct filter* filter, pa_bool_
     pa_assert(o);
     pa_assert(filter);
 
-    pa_assert_se(parent = (restore ? filter->parent_obj : filter->obj));
-
     if (is_sink_input) {
         pl = PA_SINK_INPUT(o)->proplist;
+        parent = PA_OBJECT(restore ? filter->sink_master : filter->sink);
+        if (!parent)
+            return;
         name = PA_SINK(parent)->name;
     } else {
         pl = PA_SOURCE_OUTPUT(o)->proplist;
+        parent = PA_OBJECT(restore ? filter->source_master : filter->source);
+        if (!parent)
+            return;
         name = PA_SOURCE(parent)->name;
     }
 
     pa_proplist_sets(pl, PA_PROP_FILTER_APPLY_MOVING, "1");
 
-    if (do_move(o, parent, FALSE, is_sink_input) < 0)
+    if (do_move(o, parent, false, is_sink_input) < 0)
         pa_log_info("Failed to move %s for \"%s\" to <%s>.", is_sink_input ? "sink-input" : "source-output",
                     pa_strnull(pa_proplist_gets(pl, PA_PROP_APPLICATION_NAME)), name);
     else
@@ -225,70 +311,115 @@ static void move_object_for_filter(pa_object *o, struct filter* filter, pa_bool_
     pa_proplist_unset(pl, PA_PROP_FILTER_APPLY_MOVING);
 }
 
+static void move_objects_for_filter(struct userdata *u, pa_object *o, struct filter* filter, bool restore,
+        bool is_sink_input) {
+
+    if (!should_group_filter(filter))
+        move_object_for_filter(o, filter, restore, is_sink_input);
+    else {
+        pa_source_output *so;
+        pa_sink_input *si;
+        char *g, *group;
+        uint32_t idx;
+
+        group = get_group(o, is_sink_input);
+
+        PA_IDXSET_FOREACH(so, u->core->source_outputs, idx) {
+            g = get_group(PA_OBJECT(so), false);
+
+            if (pa_streq(g, group))
+                move_object_for_filter(PA_OBJECT(so), filter, restore, false);
+
+            pa_xfree(g);
+        }
+
+        PA_IDXSET_FOREACH(si, u->core->sink_inputs, idx) {
+            g = get_group(PA_OBJECT(si), true);
+
+            if (pa_streq(g, group))
+                move_object_for_filter(PA_OBJECT(si), filter, restore, true);
+
+            pa_xfree(g);
+        }
+
+        pa_xfree(group);
+    }
+}
+
+/* Note that we assume a filter will provide at most one sink and at most one
+ * source (and at least one of either). */
 static void find_filters_for_module(struct userdata *u, pa_module *m, const char *name) {
     uint32_t idx;
     pa_sink *sink;
     pa_source *source;
-    struct filter *fltr;
+    struct filter *fltr = NULL;
 
     PA_IDXSET_FOREACH(sink, u->core->sinks, idx) {
         if (sink->module == m) {
-            pa_assert(sink->input_to_master != NULL);
+            pa_assert(pa_sink_is_filter(sink));
 
-            fltr = filter_new(name, PA_OBJECT(sink->input_to_master->sink), TRUE);
+            fltr = filter_new(name, sink->input_to_master->sink, NULL);
             fltr->module_index = m->index;
-            fltr->obj = PA_OBJECT(sink);
+            fltr->sink = sink;
 
-            pa_hashmap_put(u->filters, fltr, fltr);
+            break;
         }
     }
 
     PA_IDXSET_FOREACH(source, u->core->sources, idx) {
         if (source->module == m && !source->monitor_of) {
-            pa_assert(source->output_from_master != NULL);
+            pa_assert(pa_source_is_filter(source));
 
-            fltr = filter_new(name, PA_OBJECT(source->output_from_master->source), FALSE);
-            fltr->module_index = m->index;
-            fltr->obj = PA_OBJECT(source);
+            if (!fltr) {
+                fltr = filter_new(name, NULL, source->output_from_master->source);
+                fltr->module_index = m->index;
+                fltr->source = source;
+            } else {
+                fltr->source = source;
+                fltr->source_master = source->output_from_master->source;
+            }
 
-            pa_hashmap_put(u->filters, fltr, fltr);
+            break;
         }
     }
+
+    pa_hashmap_put(u->filters, fltr, fltr);
 }
 
-static pa_bool_t can_unload_module(struct userdata *u, uint32_t idx) {
+static bool can_unload_module(struct userdata *u, uint32_t idx) {
     void *state;
     struct filter *filter;
 
     /* Check if any other struct filters point to the same module */
     PA_HASHMAP_FOREACH(filter, u->filters, state) {
-        if (filter->module_index == idx && !nothing_attached(filter->obj, pa_sink_isinstance(filter->obj)))
-            return FALSE;
+        if (filter->module_index == idx && !nothing_attached(filter))
+            return false;
     }
 
-    return TRUE;
+    return true;
 }
 
-static pa_hook_result_t process(struct userdata *u, pa_object *o, pa_bool_t is_sink_input) {
+static pa_hook_result_t process(struct userdata *u, pa_object *o, bool is_sink_input) {
     const char *want;
-    pa_bool_t done_something = FALSE;
-
-    pa_object *parent; /* source/sink of the given source-output/sink-input */
-    const char *parent_name;
-    pa_module *module;
+    bool done_something = false;
+    pa_sink *sink = NULL;
+    pa_source *source = NULL;
+    pa_module *module = NULL;
 
     if (is_sink_input) {
-        parent = PA_OBJECT(PA_SINK_INPUT(o)->sink);
-        parent_name = PA_SINK_INPUT(o)->sink->name;
-        module = PA_SINK_INPUT(o)->sink->module;
+        sink = PA_SINK_INPUT(o)->sink;
+
+        if (sink)
+            module = sink->module;
     } else {
-        parent = PA_OBJECT(PA_SOURCE_OUTPUT(o)->source);
-        parent_name = PA_SOURCE_OUTPUT(o)->source->name;
-        module = PA_SOURCE_OUTPUT(o)->source->module;
+        source = PA_SOURCE_OUTPUT(o)->source;
+
+        if (source)
+            module = source->module;
     }
 
-    /* If there is no sink yet, we can't do much */
-    if (!parent)
+    /* If there is no sink/source yet, we can't do much */
+    if ((is_sink_input && !sink) || (!is_sink_input && !source))
         return PA_HOOK_OK;
 
     /* If the stream doesn't what any filter, then let it be. */
@@ -309,19 +440,29 @@ static pa_hook_result_t process(struct userdata *u, pa_object *o, pa_bool_t is_s
             return PA_HOOK_OK;
         }
 
-        fltr = filter_new(want, parent, is_sink_input);
+        fltr = filter_new(want, sink, source);
+
+        if (should_group_filter(fltr) && !find_paired_master(u, fltr, o, is_sink_input)) {
+            pa_log_debug("Want group filtering but don't have enough streams.");
+            return PA_HOOK_OK;
+        }
 
         if (!(filter = pa_hashmap_get(u->filters, fltr))) {
             char *args;
             pa_module *m;
 
-            args = pa_sprintf_malloc("autoloaded=1 %s_master=%s", is_sink_input ? "sink" : "source", parent_name);
+            args = pa_sprintf_malloc("autoloaded=1 %s%s %s%s",
+                    fltr->sink_master ? "sink_master=" : "",
+                    fltr->sink_master ? fltr->sink_master->name : "",
+                    fltr->source_master ? "source_master=" : "",
+                    fltr->source_master ? fltr->source_master->name : "");
+
             pa_log_debug("Loading %s with arguments '%s'", module_name, args);
 
             if ((m = pa_module_load(u->core, module_name, args))) {
                 find_filters_for_module(u, m, want);
                 filter = pa_hashmap_get(u->filters, fltr);
-                done_something = TRUE;
+                done_something = true;
             }
             pa_xfree(args);
         }
@@ -329,17 +470,17 @@ static pa_hook_result_t process(struct userdata *u, pa_object *o, pa_bool_t is_s
         pa_xfree(fltr);
 
         if (!filter) {
-            pa_log("Unable to load %s for <%s>", module_name, parent_name);
+            pa_log("Unable to load %s", module_name);
             pa_xfree(module_name);
             return PA_HOOK_OK;
         }
         pa_xfree(module_name);
 
-        if (filter->obj) {
-            /* We can move the sink_input now as the know the destination.
-             * If this isn't true, we will do it later when the sink appears. */
-            move_object_for_filter(o, filter, FALSE, is_sink_input);
-            done_something = TRUE;
+        /* We can move the stream now as we know the destination. If this
+         * isn't true, we will do it later when the sink appears. */
+        if ((is_sink_input && filter->sink) || (!is_sink_input && filter->source)) {
+            move_objects_for_filter(u, o, filter, false, is_sink_input);
+            done_something = true;
         }
     } else {
         void *state;
@@ -348,9 +489,9 @@ static pa_hook_result_t process(struct userdata *u, pa_object *o, pa_bool_t is_s
         /* We do not want to filter... but are we already filtered?
          * This can happen if an input's proplist changes */
         PA_HASHMAP_FOREACH(filter, u->filters, state) {
-            if (parent == filter->obj) {
-                move_object_for_filter(o, filter, TRUE, is_sink_input);
-                done_something = TRUE;
+            if ((is_sink_input && sink == filter->sink) || (!is_sink_input && source == filter->source)) {
+                move_objects_for_filter(u, o, filter, true, is_sink_input);
+                done_something = true;
                 break;
             }
         }
@@ -366,7 +507,7 @@ static pa_hook_result_t sink_input_put_cb(pa_core *core, pa_sink_input *i, struc
     pa_core_assert_ref(core);
     pa_sink_input_assert_ref(i);
 
-    return process(u, PA_OBJECT(i), TRUE);
+    return process(u, PA_OBJECT(i), true);
 }
 
 static pa_hook_result_t sink_input_move_finish_cb(pa_core *core, pa_sink_input *i, struct userdata *u) {
@@ -376,14 +517,14 @@ static pa_hook_result_t sink_input_move_finish_cb(pa_core *core, pa_sink_input *
     if (pa_proplist_gets(i->proplist, PA_PROP_FILTER_APPLY_MOVING))
         return PA_HOOK_OK;
 
-    return process(u, PA_OBJECT(i), TRUE);
+    return process(u, PA_OBJECT(i), true);
 }
 
 static pa_hook_result_t sink_input_proplist_cb(pa_core *core, pa_sink_input *i, struct userdata *u) {
     pa_core_assert_ref(core);
     pa_sink_input_assert_ref(i);
 
-    return process(u, PA_OBJECT(i), TRUE);
+    return process(u, PA_OBJECT(i), true);
 }
 
 static pa_hook_result_t sink_input_unlink_cb(pa_core *core, pa_sink_input *i, struct userdata *u) {
@@ -409,17 +550,17 @@ static pa_hook_result_t sink_unlink_cb(pa_core *core, pa_sink *sink, struct user
     /* If either the parent or the sink we've loaded disappears,
      * we should remove it from our hashmap */
     PA_HASHMAP_FOREACH(filter, u->filters, state) {
-        if (filter->parent_obj == PA_OBJECT(sink) || filter->obj == PA_OBJECT(sink)) {
+        if (filter->sink_master == sink || filter->sink == sink) {
             uint32_t idx;
 
             /* Attempt to rescue any streams to the parent sink as this is likely
              * the best course of action (as opposed to a generic rescue via
              * module-rescue-streams */
-            if (filter->obj == PA_OBJECT(sink)) {
+            if (filter->sink == sink) {
                 pa_sink_input *i;
 
                 PA_IDXSET_FOREACH(i, sink->inputs, idx)
-                    move_object_for_filter(PA_OBJECT(i), filter, TRUE, TRUE);
+                    move_objects_for_filter(u, PA_OBJECT(i), filter, true, true);
             }
 
             idx = filter->module_index;
@@ -427,7 +568,7 @@ static pa_hook_result_t sink_unlink_cb(pa_core *core, pa_sink *sink, struct user
             filter_free(filter);
 
             if (can_unload_module(u, idx))
-                pa_module_unload_request_by_index(u->core, idx, TRUE);
+                pa_module_unload_request_by_index(u->core, idx, true);
         }
     }
 
@@ -438,7 +579,7 @@ static pa_hook_result_t source_output_put_cb(pa_core *core, pa_source_output *o,
     pa_core_assert_ref(core);
     pa_source_output_assert_ref(o);
 
-    return process(u, PA_OBJECT(o), FALSE);
+    return process(u, PA_OBJECT(o), false);
 }
 
 static pa_hook_result_t source_output_move_finish_cb(pa_core *core, pa_source_output *o, struct userdata *u) {
@@ -448,14 +589,14 @@ static pa_hook_result_t source_output_move_finish_cb(pa_core *core, pa_source_ou
     if (pa_proplist_gets(o->proplist, PA_PROP_FILTER_APPLY_MOVING))
         return PA_HOOK_OK;
 
-    return process(u, PA_OBJECT(o), FALSE);
+    return process(u, PA_OBJECT(o), false);
 }
 
 static pa_hook_result_t source_output_proplist_cb(pa_core *core, pa_source_output *o, struct userdata *u) {
     pa_core_assert_ref(core);
     pa_source_output_assert_ref(o);
 
-    return process(u, PA_OBJECT(o), FALSE);
+    return process(u, PA_OBJECT(o), false);
 }
 
 static pa_hook_result_t source_output_unlink_cb(pa_core *core, pa_source_output *o, struct userdata *u) {
@@ -481,17 +622,17 @@ static pa_hook_result_t source_unlink_cb(pa_core *core, pa_source *source, struc
     /* If either the parent or the source we've loaded disappears,
      * we should remove it from our hashmap */
     PA_HASHMAP_FOREACH(filter, u->filters, state) {
-        if (filter->parent_obj == PA_OBJECT(source) || filter->obj == PA_OBJECT(source)) {
+        if (filter->source_master == source || filter->source == source) {
             uint32_t idx;
 
             /* Attempt to rescue any streams to the parent source as this is likely
              * the best course of action (as opposed to a generic rescue via
              * module-rescue-streams */
-            if (filter->obj == PA_OBJECT(source)) {
+            if (filter->source == source) {
                 pa_source_output *o;
 
                 PA_IDXSET_FOREACH(o, source->outputs, idx)
-                    move_object_for_filter(PA_OBJECT(o), filter, TRUE, FALSE);
+                    move_objects_for_filter(u, PA_OBJECT(o), filter, true, false);
             }
 
             idx = filter->module_index;
@@ -499,7 +640,7 @@ static pa_hook_result_t source_unlink_cb(pa_core *core, pa_source *source, struc
             filter_free(filter);
 
             if (can_unload_module(u, idx))
-                pa_module_unload_request_by_index(u->core, idx, TRUE);
+                pa_module_unload_request_by_index(u->core, idx, true);
         }
     }
 
@@ -529,16 +670,16 @@ int pa__init(pa_module *m) {
 
     u->filters = pa_hashmap_new(filter_hash, filter_compare);
 
-    u->sink_input_put_slot = pa_hook_connect(&m->core->hooks[PA_CORE_HOOK_SINK_INPUT_PUT], PA_HOOK_LATE, (pa_hook_cb_t) sink_input_put_cb, u);
-    u->sink_input_move_finish_slot = pa_hook_connect(&m->core->hooks[PA_CORE_HOOK_SINK_INPUT_MOVE_FINISH], PA_HOOK_LATE, (pa_hook_cb_t) sink_input_move_finish_cb, u);
-    u->sink_input_proplist_slot = pa_hook_connect(&m->core->hooks[PA_CORE_HOOK_SINK_INPUT_PROPLIST_CHANGED], PA_HOOK_LATE, (pa_hook_cb_t) sink_input_proplist_cb, u);
-    u->sink_input_unlink_slot = pa_hook_connect(&m->core->hooks[PA_CORE_HOOK_SINK_INPUT_UNLINK], PA_HOOK_LATE, (pa_hook_cb_t) sink_input_unlink_cb, u);
-    u->sink_unlink_slot = pa_hook_connect(&m->core->hooks[PA_CORE_HOOK_SINK_UNLINK], PA_HOOK_LATE, (pa_hook_cb_t) sink_unlink_cb, u);
-    u->source_output_put_slot = pa_hook_connect(&m->core->hooks[PA_CORE_HOOK_SOURCE_OUTPUT_PUT], PA_HOOK_LATE, (pa_hook_cb_t) source_output_put_cb, u);
-    u->source_output_move_finish_slot = pa_hook_connect(&m->core->hooks[PA_CORE_HOOK_SOURCE_OUTPUT_MOVE_FINISH], PA_HOOK_LATE, (pa_hook_cb_t) source_output_move_finish_cb, u);
-    u->source_output_proplist_slot = pa_hook_connect(&m->core->hooks[PA_CORE_HOOK_SOURCE_OUTPUT_PROPLIST_CHANGED], PA_HOOK_LATE, (pa_hook_cb_t) source_output_proplist_cb, u);
-    u->source_output_unlink_slot = pa_hook_connect(&m->core->hooks[PA_CORE_HOOK_SOURCE_OUTPUT_UNLINK], PA_HOOK_LATE, (pa_hook_cb_t) source_output_unlink_cb, u);
-    u->source_unlink_slot = pa_hook_connect(&m->core->hooks[PA_CORE_HOOK_SOURCE_UNLINK], PA_HOOK_LATE, (pa_hook_cb_t) source_unlink_cb, u);
+    pa_module_hook_connect(m, &m->core->hooks[PA_CORE_HOOK_SINK_INPUT_PUT], PA_HOOK_LATE, (pa_hook_cb_t) sink_input_put_cb, u);
+    pa_module_hook_connect(m, &m->core->hooks[PA_CORE_HOOK_SINK_INPUT_MOVE_FINISH], PA_HOOK_LATE, (pa_hook_cb_t) sink_input_move_finish_cb, u);
+    pa_module_hook_connect(m, &m->core->hooks[PA_CORE_HOOK_SINK_INPUT_PROPLIST_CHANGED], PA_HOOK_LATE, (pa_hook_cb_t) sink_input_proplist_cb, u);
+    pa_module_hook_connect(m, &m->core->hooks[PA_CORE_HOOK_SINK_INPUT_UNLINK], PA_HOOK_LATE, (pa_hook_cb_t) sink_input_unlink_cb, u);
+    pa_module_hook_connect(m, &m->core->hooks[PA_CORE_HOOK_SINK_UNLINK], PA_HOOK_LATE-1, (pa_hook_cb_t) sink_unlink_cb, u);
+    pa_module_hook_connect(m, &m->core->hooks[PA_CORE_HOOK_SOURCE_OUTPUT_PUT], PA_HOOK_LATE, (pa_hook_cb_t) source_output_put_cb, u);
+    pa_module_hook_connect(m, &m->core->hooks[PA_CORE_HOOK_SOURCE_OUTPUT_MOVE_FINISH], PA_HOOK_LATE, (pa_hook_cb_t) source_output_move_finish_cb, u);
+    pa_module_hook_connect(m, &m->core->hooks[PA_CORE_HOOK_SOURCE_OUTPUT_PROPLIST_CHANGED], PA_HOOK_LATE, (pa_hook_cb_t) source_output_proplist_cb, u);
+    pa_module_hook_connect(m, &m->core->hooks[PA_CORE_HOOK_SOURCE_OUTPUT_UNLINK], PA_HOOK_LATE, (pa_hook_cb_t) source_output_unlink_cb, u);
+    pa_module_hook_connect(m, &m->core->hooks[PA_CORE_HOOK_SOURCE_UNLINK], PA_HOOK_LATE-1, (pa_hook_cb_t) source_unlink_cb, u);
 
     pa_modargs_free(ma);
 
@@ -561,27 +702,6 @@ void pa__done(pa_module *m) {
     if (!(u = m->userdata))
         return;
 
-    if (u->sink_input_put_slot)
-        pa_hook_slot_free(u->sink_input_put_slot);
-    if (u->sink_input_move_finish_slot)
-        pa_hook_slot_free(u->sink_input_move_finish_slot);
-    if (u->sink_input_proplist_slot)
-        pa_hook_slot_free(u->sink_input_proplist_slot);
-    if (u->sink_input_unlink_slot)
-        pa_hook_slot_free(u->sink_input_unlink_slot);
-    if (u->sink_unlink_slot)
-        pa_hook_slot_free(u->sink_unlink_slot);
-    if (u->source_output_put_slot)
-        pa_hook_slot_free(u->source_output_put_slot);
-    if (u->source_output_move_finish_slot)
-        pa_hook_slot_free(u->source_output_move_finish_slot);
-    if (u->source_output_proplist_slot)
-        pa_hook_slot_free(u->source_output_proplist_slot);
-    if (u->source_output_unlink_slot)
-        pa_hook_slot_free(u->source_output_unlink_slot);
-    if (u->source_unlink_slot)
-        pa_hook_slot_free(u->source_unlink_slot);
-
     if (u->housekeeping_time_event)
         u->core->mainloop->time_free(u->housekeeping_time_event);
 
@@ -589,11 +709,11 @@ void pa__done(pa_module *m) {
         struct filter *f;
 
         while ((f = pa_hashmap_steal_first(u->filters))) {
-            pa_module_unload_request_by_index(u->core, f->module_index, TRUE);
+            pa_module_unload_request_by_index(u->core, f->module_index, true);
             filter_free(f);
         }
 
-        pa_hashmap_free(u->filters, NULL, NULL);
+        pa_hashmap_free(u->filters);
     }
 
     pa_xfree(u);
